from datetime import date, datetime
from typing import Optional

from tikkie2._session import ApiSession, get_session
from tikkie2._util import JsonDict, format_date
from tikkie2.v2.models import GetAllPaymentRequestsResponse, PaymentRequest


def create(
    *,
    description: str,
    amount_in_cents: Optional[int] = None,
    expiry_date: Optional[date] = None,
    reference_id: Optional[str] = None,
    app_token: Optional[str] = None,
    api_key: Optional[str] = None,
    session: Optional[ApiSession] = None,
) -> PaymentRequest:
    """
    https://developer.abnamro.com/api-products/tikkie/reference-documentation#operation/createPaymentRequest

    :param description: Description of the payment request, which the payer or payers see.
    :param amount_in_cents: Amount in cents of the payment request (euros). If this value is not specified, an open payment request is created.
    :param expiry_date: Date after the payment request expires and cannot be paid
    :param reference_id: ID for the reference of the API consumer.
    :param app_token: The App Token generated in the Tikkie Business Portal.
    :param api_key: The API Key obtained from the ABN developer portal.
    :param session: An optional API session class. If not passed the global API session will be used
    """
    s = session if session else get_session()
    payload: JsonDict = {"description": description}
    if amount_in_cents is not None:
        payload["amountInCents"] = amount_in_cents
    if expiry_date is not None:
        payload["expiryDate"] = format_date(expiry_date)
    if reference_id is not None:
        payload["referenceId"] = reference_id

    response = s.post("paymentrequests", json=payload, app_token=app_token, api_key=api_key)
    return PaymentRequest.parse_raw(response.content)


def get(
    payment_request_token: str,
    app_token: Optional[str] = None,
    api_key: Optional[str] = None,
    session: Optional[ApiSession] = None,
) -> PaymentRequest:
    """
    https://developer.abnamro.com/api-products/tikkie/reference-documentation#operation/getPaymentRequest

    :param payment_request_token: Token identifying the payment request.
    :param app_token: The App Token generated in the Tikkie Business Portal.
    :param api_key: The API Key obtained from the ABN developer portal.
    :param session: An optional API session class. If not passed the global API session will be used
    """
    s = session if session else get_session()
    response = s.get(
        f"paymentrequests/{payment_request_token}",
        app_token=app_token,
        api_key=api_key,
    )
    return PaymentRequest.parse_raw(response.content)


def get_all(
    *,
    page_size: int,
    page_number: int,
    from_date_time: Optional[datetime] = None,
    to_date_time: Optional[datetime] = None,
    app_token: Optional[str] = None,
    api_key: Optional[str] = None,
    session: Optional[ApiSession] = None,
) -> GetAllPaymentRequestsResponse:
    """
    https://developer.abnamro.com/api-products/tikkie/reference-documentation#operation/getPaymentRequestList

    :param page_size: Number of the page to be returned.
    :param page_number: Number of items on a page.
    :param from_date_time:The time you start searching for items. Refers to the creation date for payment requests and iDeal QRs, and refers to the bundle date for transaction bundles.
    :param to_date_time: The time you stop searching for items. Refers to the creation date for payment requests and iDeal QRs, and refers to the bundle date for transaction bundles.
    :param app_token: The App Token generated in the Tikkie Business Portal.
    :param api_key: The API Key obtained from the ABN developer portal.
    :param session: An optional API session class. If not passed the global API session will be used
    """
    s = session if session else get_session()
    params: JsonDict = {
        "pageSize": page_size,
        "pageNumber": page_number,
    }
    if from_date_time is not None:
        params["fromDateTime"] = from_date_time
    if to_date_time is not None:
        params["toDateTime"] = to_date_time

    response = s.get(
        "paymentrequests",
        params=params,
        app_token=app_token,
        api_key=api_key,
    )
    return GetAllPaymentRequestsResponse.parse_raw(response.content)



import os
from functools import lru_cache
from typing import Any, Optional

import requests
from requests import Response

from tikkie2._version import VERSION
from tikkie2.errors import BaseError

API_KEY_ENV = "TIKKIE_API_KEY"
APP_TOKEN_ENV = "TIKKIE_APP_TOKEN"
API_URL_ENV = "TIKKIE_API_URL"
PRODUCTION_ENV = "TIKKIE_PRODUCTION"


class ApiSession(requests.Session):
    def __init__(
        self,
        *,
        api_key: str,
        app_token: Optional[str] = None,
        api_url: Optional[str] = None,
        production: bool = False,
    ):
        super().__init__()
        if api_url is None:
            api_url = "https://api.abnamro.com" if production else "https://api-sandbox.abnamro.com"

        self.api_url = api_url
        self.app_token = app_token
        self.api_key = api_key
        self.production = production

        self.headers["User-Agent"] = f"Python Tikkie API/{VERSION}"

    def prepare_request(self, request: Any) -> Any:
        request.url = "/".join(s.strip("/") for s in [self.api_url, "v2/tikkie", request.url])
        return super().prepare_request(request)

    def request(
        self,
        *args: Any,
        params: Any = None,
        app_token: Optional[str] = None,
        api_key: Optional[str] = None,
        **kwargs: Any,
    ) -> Response:
        dict_params = params or {}
        headers = kwargs.pop("headers", {})

        api_key = api_key or self.api_key
        app_token = app_token or self.app_token

        if not api_key:
            raise Exception("API Key is not set")

        if not app_token:
            raise Exception("App Token is not set")

        headers["API-Key"] = api_key
        headers["X-App-Token"] = app_token

        # Manually convert the json objects as we want to use a custom encoder
        # if "json" in kwargs:
        #     payload = kwargs.pop("json")
        #     kwargs["data"] = json.dumps(payload, cls=JSONEncoder)
        #     headers["Content-Type"] = "application/json"

        r = super().request(*args, params=dict_params, headers=headers, **kwargs)  # type: ignore
        if not r.ok:
            raise BaseError.from_response(r)
        return r


# The global session object
_session: Optional[ApiSession] = None


def _read_from_os(var: Optional[str], env: str) -> Optional[str]:
    if var is None:
        var = os.environ.get(env)
    return var


def _read_from_os_as_bool(var: Optional[bool], env: str) -> bool:
    if var is None:
        var = os.environ.get(env, "")
        return var.lower() in {"1", "on", "true", "yes"}
    return var


@lru_cache()
def get_session() -> ApiSession:
    global _session
    if _session is None:
        configure()
    return _session  # type: ignore


def configure(
    *,
    app_token: Optional[str] = None,
    api_key: Optional[str] = None,
    api_url: Optional[str] = None,
    production: Optional[bool] = True,
) -> None:
    global _session

    api_key = _read_from_os(api_key, API_KEY_ENV)
    app_token = _read_from_os(app_token, APP_TOKEN_ENV)
    api_url = _read_from_os(api_url, API_URL_ENV)
    production = _read_from_os_as_bool(production, PRODUCTION_ENV)

    get_session.cache_clear()

    _session = ApiSession(
        app_token=app_token, api_url=api_url, api_key=api_key, production=production
    )



from . import (  # noqa
    ideal_qr,
    ideal_qr_notification,
    payment,
    payment_request,
    payment_request_notification,
    refund,
    sandbox_app,
    transaction_bundle,
    transactions_notifications,
)



from datetime import date, datetime
from enum import Enum
from typing import List, Optional

from pydantic import Field

from tikkie2._util import CamelCaseModel


class PaymentRequestStatus(Enum):
    OPEN = "OPEN"
    CLOSED = "CLOSED"
    EXPIRED = "EXPIRED"
    MAX_YIELD_REACHED = "MAX_YIELD_REACHED"
    MAX_SUCCESSFUL_PAYMENTS_REACHED = "MAX_SUCCESSFUL_PAYMENTS_REACHED"


class PaymentRequest(CamelCaseModel):
    payment_request_token: str
    description: str
    url: str
    expiry_date: date
    created_date_time: datetime
    status: PaymentRequestStatus
    number_of_payments: int
    total_amount_paid_in_cents: int
    amount_in_cents: Optional[int] = None
    reference_id: Optional[str] = None


class GetAllPaymentRequestsResponse(CamelCaseModel):
    payment_requests: List[PaymentRequest]
    total_element_count: int


class Refund(CamelCaseModel):
    refund_token: str
    amount_in_cents: int
    description: str
    created_date_time: datetime
    status: str  # TODO: Fill all statuses (PENDING, ?)
    reference_id: Optional[str] = None


class Payment(CamelCaseModel):
    payment_token: str
    tikkie_id: int
    counter_party_name: str
    counter_party_account_number: str
    amount_in_cents: int
    description: str
    created_date_time: datetime
    refunds: List[Refund]


class GetAllPaymentsResponse(CamelCaseModel):
    payments: List[Payment]
    total_element_count: int


class SubscribeResponse(CamelCaseModel):
    subscription_id: str


class CreateSandboxApplicationResponse(CamelCaseModel):
    app_token: str


class File(CamelCaseModel):
    type: str
    url: str


class Bundle(CamelCaseModel):
    bundle_id: str
    reference: str
    files: List[File]
    start_date_time: datetime
    end_date_time: datetime


class GetAllBundlesResponse(CamelCaseModel):
    bundles: List[Bundle]
    total_element_count: int


class IdealQr(CamelCaseModel):
    qr_id: str
    url: str
    description: str
    amount_changeable: bool
    created_date_time: datetime
    expiry_date_time: datetime
    single_pay: bool
    number_of_payments: int
    total_amount_paid_in_cents: int
    status: str  # TODO: Find all statuses (OPEN, ?)
    image_size: int
    reference_id: Optional[str] = None
    amount_in_cents: Optional[int] = None
    max_amount_in_cents: Optional[int] = None
    min_amount_in_cents: Optional[int] = None


class GetAllIdealQrResponse(CamelCaseModel):
    ideal_qrs: List[IdealQr] = Field(None, alias="idealQRs")
    total_element_count: int


class IdealQrPayment(CamelCaseModel):
    payment_token: str
    tikkie_id: int
    counter_party_name: str
    counter_party_account_number: str
    amount_in_cents: int
    description: str
    created_date_time: datetime


class GetAllIdealQrPaymentsResponse(CamelCaseModel):
    payments: List[IdealQrPayment]
    total_element_count: int



from datetime import datetime
from typing import Optional

from tikkie2._session import ApiSession, get_session
from tikkie2._util import JsonDict
from tikkie2.v2.models import Bundle, GetAllBundlesResponse


def get(
    bundle_id: str,
    app_token: Optional[str] = None,
    api_key: Optional[str] = None,
    session: Optional[ApiSession] = None,
) -> Bundle:
    """
    Retrieves a specific bundle based on a bundle identifier. The
    application must have transactions permission to complete this
    request.

    https://developer.abnamro.com/api-products/tikkie/reference-documentation#operation/getBundle

    :param bundle_id: Identifier for the bundle.
    :param app_token: The App Token generated in the Tikkie Business Portal.
    :param api_key: The API Key obtained from the ABN developer portal.
    :param session: An optional API session class. If not passed the global API session will be used
    """
    s = session if session else get_session()
    response = s.get(
        f"transactionbundles/{bundle_id}",
        app_token=app_token,
        api_key=api_key,
    )
    return Bundle.parse_raw(response.content)


def get_all(
    *,
    page_size: int,
    page_number: int,
    from_date_time: Optional[datetime] = None,
    to_date_time: Optional[datetime] = None,
    app_token: Optional[str] = None,
    api_key: Optional[str] = None,
    session: Optional[ApiSession] = None,
) -> GetAllBundlesResponse:
    """
    Retrieves a list of bundles based on the parameters provided. The
    application must have transactions permission to complete this request.

    https://developer.abnamro.com/api-products/tikkie/reference-documentation#operation/getBundleList

    :param page_size: Number of the page to be returned.
    :param page_number: Number of items on a page.
    :param from_date_time:The time you start searching for items. Refers to the creation date for payment requests and iDeal QRs, and refers to the bundle date for transaction bundles.
    :param to_date_time: The time you stop searching for items. Refers to the creation date for payment requests and iDeal QRs, and refers to the bundle date for transaction bundles.
    :param app_token: The App Token generated in the Tikkie Business Portal.
    :param api_key: The API Key obtained from the ABN developer portal.
    :param session: An optional API session class. If not passed the global API session will be used
    """
    s = session if session else get_session()
    params: JsonDict = {
        "pageSize": page_size,
        "pageNumber": page_number,
    }
    if from_date_time is not None:
        params["fromDateTime"] = from_date_time
    if to_date_time is not None:
        params["toDateTime"] = to_date_time

    response = s.get(
        "transactionbundles",
        params=params,
        app_token=app_token,
        api_key=api_key,
    )
    return GetAllBundlesResponse.parse_raw(response.content)



import logging
from typing import Any

from requests import Response


class BaseError(Exception):
    response: Response
    code: str
    message: str
    reference: str
    trace_id: str
    status: int

    def __init__(
        self,
        message: str,
        *,
        response: Response,
        code: str,
        reference: str,
        trace_id: str,
        status: int,
    ):
        super().__init__(message)
        self.response = response
        self.code = code
        self.reference = reference
        self.trace_id = trace_id
        self.status = status

    @classmethod
    def from_response(cls, response: Any) -> "BaseError":
        payload = response.json()

        # Tikkie returns multiple errors, but that makes for an weird error
        # handling system. So let's just use the first error
        error = payload["errors"][0]

        for ErrorClass in BaseError.__subclasses__():
            if error["code"] == ErrorClass.code:
                return ErrorClass(
                    error["message"],
                    response=response,
                    code=error["code"],
                    reference=error["reference"],
                    trace_id=error["traceId"],
                    status=error["status"],
                )

        # Not implemented exception found, just raise the base class
        logging.warning(
            f"Unknow Tikkie response found: code={error['code']!r} message={error['message']!r}"
        )
        return cls(
            error["message"],
            response=response,
            code=error["code"],
            reference=error["reference"],
            trace_id=error["traceId"],
            status=error["status"],
        )


class AmountChangeableInvalid(BaseError):
    code = "AMOUNT_CHANGEABLE_INVALID"


class AmountChangeableParametersInvalid(BaseError):
    code = "AMOUNT_CHANGEABLE_PARAMETERS_INVALID"


class AmountInCentsInvalid(BaseError):
    code = "AMOUNT_IN_CENTS_INVALID"


class AmountInCentsMissing(BaseError):
    code = "AMOUNT_IN_CENTS_MISSING"


class AppTokenAlreadyInUse(BaseError):
    code = "APP_TOKEN_ALREADY_IN_USE"


class AppTokenDisabled(BaseError):
    code = "APP_TOKEN_DISABLED"


class AppTokenInvalid(BaseError):
    code = "APP_TOKEN_INVALID"


class BundleIDInvalid(BaseError):
    code = "BUNDLE_ID_INVALID"


class BundleNotFound(BaseError):
    code = "BUNDLE_NOT_FOUND"


class BundledPayoutNotEnabled(BaseError):
    code = "BUNDLED_PAYOUT_NOT_ENABLED"


class DescriptionInvalid(BaseError):
    code = "DESCRIPTION_INVALID"


class DescriptionMaxLengthExceeded(BaseError):
    code = "DESCRIPTION_MAX_LENGTH_EXCEEDED"


class DescriptionMissing(BaseError):
    code = "DESCRIPTION_MISSING"


class ExpiryDateInvalid(BaseError):
    code = "EXPIRY_DATE_INVALID"


class ExpiryDateNotAllowed(BaseError):
    code = "EXPIRY_DATE_NOT_ALLOWED"


class ExpiryDateTimeInvalid(BaseError):
    code = "EXPIRY_DATE_TIME_INVALID"


class ExpiryDateTimeMissing(BaseError):
    code = "EXPIRY_DATE_TIME_MISSING"


class ExpiryDateTimeNotAllowed(BaseError):
    code = "EXPIRY_DATE_TIME_NOT_ALLOWED"


class FromDateTimeInvalid(BaseError):
    code = "FROM_DATE_TIME_INVALID"


class IdealQrNotFound(BaseError):
    code = "IDEAL_QR_NOT_FOUND"


class IdealqrForbidden(BaseError):
    code = "IDEALQR_FORBIDDEN"


class ImageSizeInvalid(BaseError):
    code = "IMAGE_SIZE_INVALID"


class ImageSizeTooLarge(BaseError):
    code = "IMAGE_SIZE_TOO_LARGE"


class ImageSizeTooSmall(BaseError):
    code = "IMAGE_SIZE_TOO_SMALL"


class InternalServerError(BaseError):
    code = "INTERNAL_SERVER_ERROR"


class MaxAmountInCentsInvalid(BaseError):
    code = "MAX_AMOUNT_IN_CENTS_INVALID"


class MaxAmountInCentsMissing(BaseError):
    code = "MAX_AMOUNT_IN_CENTS_MISSING"


class MinAmountInCentsInvalid(BaseError):
    code = "MIN_AMOUNT_IN_CENTS_INVALID"


class PageNumberInvalid(BaseError):
    code = "PAGE_NUMBER_INVALID"


class PageNumberMissing(BaseError):
    code = "PAGE_NUMBER_MISSING"


class PageSizeInvalid(BaseError):
    code = "PAGE_SIZE_INVALID"


class PageSizeMissing(BaseError):
    code = "PAGE_SIZE_MISSING"


class PageSizeTooLarge(BaseError):
    code = "PAGE_SIZE_TOO_LARGE"


class PaymentNotFound(BaseError):
    code = "PAYMENT_NOT_FOUND"


class PaymentRequestForbidden(BaseError):
    code = "PAYMENT_REQUEST_FORBIDDEN"


class PaymentRequestMaxAmountExceeded(BaseError):
    code = "PAYMENT_REQUEST_MAX_AMOUNT_EXCEEDED"


class PaymentRequestNotFound(BaseError):
    code = "PAYMENT_REQUEST_NOT_FOUND"


class PaymentRequestTokenInvalid(BaseError):
    code = "PAYMENT_REQUEST_TOKEN_INVALID"


class PaymentTokenInvalid(BaseError):
    code = "PAYMENT_TOKEN_INVALID"


class QrIDInvalid(BaseError):
    code = "QR_ID_INVALID"


class ReferenceIDInvalid(BaseError):
    code = "REFERENCE_ID_INVALID"


class ReferenceIDMaxLengthExceeded(BaseError):
    code = "REFERENCE_ID_MAX_LENGTH_EXCEEDED"


class RefundAmountIsTooHigh(BaseError):
    code = "REFUND_AMOUNT_IS_TOO_HIGH"


class RefundForbidden(BaseError):
    code = "REFUND_FORBIDDEN"


class RefundNotFound(BaseError):
    code = "REFUND_NOT_FOUND"


class RefundTokenInvalid(BaseError):
    code = "REFUND_TOKEN_INVALID"


class SinglePayInvalid(BaseError):
    code = "SINGLE_PAY_INVALID"


class TimeRangeInvalid(BaseError):
    code = "TIME_RANGE_INVALID"


class ToDateTimeInvalid(BaseError):
    code = "TO_DATE_TIME_INVALID"


class TransactionsForbidden(BaseError):
    code = "TRANSACTIONS_FORBIDDEN"


class URLDisallowed(BaseError):
    code = "URL_DISALLOWED"


class URLInvalid(BaseError):
    code = "URL_INVALID"


class URLMissing(BaseError):
    code = "URL_MISSING"



from datetime import datetime
from typing import Optional

from tikkie2._session import ApiSession, get_session
from tikkie2._util import JsonDict, format_datetime
from tikkie2.v2.models import (
    GetAllIdealQrPaymentsResponse,
    GetAllIdealQrResponse,
    IdealQr,
    IdealQrPayment,
)


def create_non_changeable(
    *,
    description: str,
    expiry_date_time: datetime,
    amount_in_cents: int,
    reference_id: Optional[str] = None,
    single_pay: Optional[bool] = None,
    image_size: Optional[int] = None,
    app_token: Optional[str] = None,
    api_key: Optional[str] = None,
    session: Optional[ApiSession] = None,
) -> IdealQr:
    """
    https://developer.abnamro.com/api-products/tikkie/reference-documentation#tag/iDeal-QR

    :param description: Description of the iDeal QR request, which the payer or payers see.
    :param expiry_date_time: The date and time at which the iDeal QR expires and can no longer be paid. The expiration is accurate up to minutes.
    :param amount_in_cents: The amount to pay for the iDeal QR.
    :param reference_id: ID for the reference of the API consumer.
    :param single_pay: Indicates whether the iDeal QR can be paid once, or multiple times.
    :param image_size: Size of the QR code image in pixels. It can be between 100 and 2000 pixels inclusive.
    :param app_token: The App Token generated in the Tikkie Business Portal.
    :param api_key: The API Key obtained from the ABN developer portal.
    :param session: An optional API session class. If not passed the global API session will be used
    """
    s = session if session else get_session()
    payload = {
        "description": description,
        "amountInCents": amount_in_cents,
        "expiryDateTime": format_datetime(expiry_date_time),
        "amountChangeable": False,
    }
    if reference_id is not None:
        payload["referenceId"] = reference_id
    if single_pay is not None:
        payload["singlePay"] = single_pay
    if image_size is not None:
        payload["imageSize"] = image_size
    if expiry_date_time is not None:
        payload["expiryDateTime"] = format_datetime(expiry_date_time)

    response = s.post("idealqrs", app_token=app_token, api_key=api_key, json=payload)
    return IdealQr.parse_raw(response.content)


def create_changeable(
    *,
    description: str,
    expiry_date_time: datetime,
    max_amount_in_cents: int,
    reference_id: str = None,
    single_pay: bool = None,
    image_size: int = None,
    min_amount_in_cents: int = None,
    app_token: Optional[str] = None,
    api_key: Optional[str] = None,
    session: Optional[ApiSession] = None,
) -> IdealQr:
    """
    https://developer.abnamro.com/api-products/tikkie/reference-documentation#tag/iDeal-QR

    :param description: Description of the iDeal QR request, which the payer or payers see.
    :param expiry_date_time: The date and time at which the iDeal QR expires and can no longer be paid. The expiration is accurate up to minutes.
    :param max_amount_in_cents: The maximum amount the user can enter for the iDeal QR.
    :param reference_id: ID for the reference of the API consumer.
    :param single_pay: Indicates whether the iDeal QR can be paid once, or multiple times.
    :param image_size: Size of the QR code image in pixels. It can be between 100 and 2000 pixels inclusive.
    :param min_amount_in_cents: The minimum amount the user can enter for the iDeal QR.
    :param app_token: The App Token generated in the Tikkie Business Portal.
    :param api_key: The API Key obtained from the ABN developer portal.
    :param session: An optional API session class. If not passed the global API session will be used
    """
    s = session if session else get_session()
    payload = {
        "description": description,
        "expiryDateTime": format_datetime(expiry_date_time),
        "amountChangeable": True,
        "maxAmountInCents": max_amount_in_cents,
    }
    if reference_id is not None:
        payload["referenceId"] = reference_id
    if single_pay is not None:
        payload["singlePay"] = single_pay
    if image_size is not None:
        payload["imageSize"] = image_size
    if expiry_date_time is not None:
        payload["expiryDateTime"] = format_datetime(expiry_date_time)
    if min_amount_in_cents is not None:
        payload["minAmountInCents"] = min_amount_in_cents

    response = s.post("idealqrs", app_token=app_token, api_key=api_key, json=payload)
    return IdealQr.parse_raw(response.content)


def get(
    *,
    qr_id: str,
    app_token: Optional[str] = None,
    api_key: Optional[str] = None,
    session: Optional[ApiSession] = None,
) -> IdealQr:
    """
    Retrieves a specific iDeal QR based on a token value. The
    application must have iDeal QR permission to complete this
    request.

    :param qr_id: Identifier for iDeal QR.
    :param app_token: The App Token generated in the Tikkie Business Portal.
    :param api_key: The API Key obtained from the ABN developer portal.
    :param session: An optional API session class. If not passed the global API session will be used

    https://developer.abnamro.com/api-products/tikkie/reference-documentation#operation/createPaymentRequest
    """
    s = session if session else get_session()
    response = s.get(f"idealqrs/{qr_id}", app_token=app_token, api_key=api_key)
    return IdealQr.parse_raw(response.content)


def get_all(
    *,
    page_size: int,
    page_number: int,
    from_date_time: Optional[datetime] = None,
    to_date_time: Optional[datetime] = None,
    app_token: Optional[str] = None,
    api_key: Optional[str] = None,
    session: Optional[ApiSession] = None,
) -> GetAllIdealQrResponse:
    """
    Retrieves a list of iDeal QRs based on the parameters provided.
    The application must have iDeal QR permission to complete this
    request.

    https://developer.abnamro.com/api-products/tikkie/reference-documentation#operation/getPaymentRequest

    :param page_size: Number of the page to be returned.
    :param page_number: Number of items on a page.
    :param from_date_time:The time you start searching for items. Refers to the creation date for payment requests and iDeal QRs, and refers to the bundle date for transaction bundles.
    :param to_date_time: The time you stop searching for items. Refers to the creation date for payment requests and iDeal QRs, and refers to the bundle date for transaction bundles.
    :param app_token: The App Token generated in the Tikkie Business Portal.
    :param api_key: The API Key obtained from the ABN developer portal.
    :param session: An optional API session class. If not passed the global API session will be used
    """
    s = session if session else get_session()
    params: JsonDict = {
        "pageSize": page_size,
        "pageNumber": page_number,
    }
    if from_date_time is not None:
        params["fromDateTime"] = from_date_time
    if to_date_time is not None:
        params["toDateTime"] = to_date_time

    response = s.get(
        "idealqrs",
        params=params,
        app_token=app_token,
        api_key=api_key,
    )
    return GetAllIdealQrResponse.parse_raw(response.content)


def get_all_payments(
    *,
    qr_id: str,
    page_size: int,
    page_number: int,
    from_date_time: Optional[datetime] = None,
    to_date_time: Optional[datetime] = None,
    app_token: Optional[str] = None,
    api_key: Optional[str] = None,
    session: Optional[ApiSession] = None,
) -> GetAllIdealQrPaymentsResponse:
    """
    Retrieves payments made against a specific iDeal QR based on the
    parameters provided. The application must have iDeal QR
    permission to complete this request.

    https://developer.abnamro.com/api-products/tikkie/reference-documentation#operation/getIDealQR

    :param page_size: Number of the page to be returned.
    :param page_number: Number of items on a page.
    :param from_date_time:The time you start searching for items. Refers to the creation date for payment requests and iDeal QRs, and refers to the bundle date for transaction bundles.
    :param to_date_time: The time you stop searching for items. Refers to the creation date for payment requests and iDeal QRs, and refers to the bundle date for transaction bundles.
    :param app_token: The App Token generated in the Tikkie Business Portal.
    :param api_key: The API Key obtained from the ABN developer portal.
    :param session: An optional API session class. If not passed the global API session will be used
    """
    s = session if session else get_session()
    params: JsonDict = {
        "pageSize": page_size,
        "pageNumber": page_number,
    }
    if from_date_time is not None:
        params["fromDateTime"] = from_date_time
    if to_date_time is not None:
        params["toDateTime"] = to_date_time

    response = s.get(
        f"idealqrs/{qr_id}/payments",
        params=params,
        app_token=app_token,
        api_key=api_key,
    )
    return GetAllIdealQrPaymentsResponse.parse_raw(response.content)


def get_payment(
    *,
    qr_id: str,
    payment_token: str,
    app_token: Optional[str] = None,
    api_key: Optional[str] = None,
    session: Optional[ApiSession] = None,
) -> IdealQrPayment:
    """
    Retrieves details of a specific payment made against a specific
    iDeal QR based on the token and ID value. The application must
    have iDeal QR permission to complete this request.

    https://developer.abnamro.com/api-products/tikkie/reference-documentation#operation/getIDealQRPayment

    :param qr_id: Number of the page to be returned.
    :param payment_token: Number of items on a page.
    :param app_token: The App Token generated in the Tikkie Business Portal.
    :param api_key: The API Key obtained from the ABN developer portal.
    :param session: An optional API session class. If not passed the global API session will be used
    """
    s = session if session else get_session()
    response = s.get(
        f"idealqrs/{qr_id}/payments/{payment_token}",
        app_token=app_token,
        api_key=api_key,
    )
    return IdealQrPayment.parse_raw(response.content)
